<?php

namespace June\Components\Api\Client;

use GuzzleHttp\Exception\GuzzleException;
use June\Components\Logger\Helper\LoggerHelper;
use Shopware\Core\System\SystemConfig\SystemConfigService;

class JuneApiClient {

    const JUNE_TRANSFER_TYPE = "June.config.transfertype";

    /**
     * @var SystemConfigService
     */
    private SystemConfigService $configService;

    private LoggerHelper $loggerHelper;

    public function __construct
    (
        SystemConfigService $configService,
        LoggerHelper $loggerHelper
    )
    {
        $this->configService = $configService;
        $this->loggerHelper = $loggerHelper;
    }

    public function sendData($data, $url, $auth) {
        if($this->configService->get(self::JUNE_TRANSFER_TYPE)) {
            $this->socketRequest($data, $url);
        } else {
            $this->httpRequest($data, $url, $auth);
        }
    }

    private function socketRequest($data, $url) {
        $endpoint = $url;
        $postData = json_encode($data);
        $endpointParts = parse_url($endpoint);
        $endpointParts['path'] = $endpointParts['path'] ?? '/';
        $endpointParts['port'] = $endpointParts['port'] ?? $endpointParts['scheme'] === 'https' ? 443 : 80;

        $contentLength = strlen($postData);

        $request = "POST {$endpointParts['path']} HTTP/1.1\r\n";
        $request .= "Host: {$endpointParts['host']}\r\n";
        $request .= "Content-Type: application/json\r\n";
        $request .= "Content-Length: {$contentLength}\r\n";
        $request .= "Connection: Close\r\n\r\n";
        $request .= $postData;

        $prefix = substr($endpoint, 0, 8) === 'https://' ? 'tls://' : ''; // maybe use ssl?


        $this->loggerHelper->logSocket("--- Step 1: Connection Info ---");
        $this->loggerHelper->logSocket("Endpoint: ".$prefix.$endpointParts['host']);
        $this->loggerHelper->logSocket("Port: ".$endpointParts['port']);

        try {
            $this->loggerHelper->logSocket("--- Step 2: Start Connection ---");
            $socket = fsockopen($prefix.$endpointParts['host'], $endpointParts['port'], $errNo, $errStr, 1);

            if(!$socket || $errNo || $errStr) {
                $this->loggerHelper->logSocket("Connection Failed {$errNo}: {$errStr}");
            } else {
                $this->loggerHelper->logSocket("Connection is established");

                fwrite($socket, $request);

                if(fflush($socket)) {
                    $this->loggerHelper->logSocket("POST Data Success");
                } else {
                    $this->loggerHelper->logSocket("POST Data Failed");
                }
            }
            fclose($socket);
            $this->loggerHelper->logSocket("--- Step 3: Close Connection ---");

        } catch(\Exception $ex) {
            $this->loggerHelper->logSocket("ERROR: ".$ex->getMessage());
        }
    }

    public function httpRequest($data, $url, $auth) {
        $header = ['Accept' => 'application/json'];
        if ($auth != null) {
            $header['Authorization'] = 'Bearer ' . $auth;
        }
        $options["headers"] = $header;
        $options['json'] = $data;
        $client = new \GuzzleHttp\Client(["timeout" => 2]);

        try {
            $client->post($url, $options);
        } catch (GuzzleException $e) {
            $this->loggerHelper->logError($e->getMessage(), 'httpRequest');
        }
    }
}
